"""
This script shows how MALA can be used to run inference.

Can be used for both aluminium and beryllium data.
"""
import mala
from mala.common.parallelizer import get_rank
from ase.io import read
import numpy as np

# Set the path to your trained model and load the run.
# For inference (especially the large scale inferences), the usage of MPI is
# recommended.
surrogate_model_folder = "PATH_TO_YOUR_TRAINING_FOLDER"
parameters = mala.Parameters.load_from_file(surrogate_model_folder +
                                            "training_run1.params.json")
parameters.use_gpu = False
parameters.use_mpi = True
network = mala.Network.load_from_file(parameters, surrogate_model_folder +
                                      "training_run1.network.pth")
iscaler = mala.DataScaler.load_from_file(surrogate_model_folder +
                                         "training_run1.iscaler.pkl")
oscaler = mala.DataScaler.load_from_file(surrogate_model_folder +
                                         "training_run1.oscaler.pkl")

# Set the path to your pseudopotential folder, i.e., the folder where the SAME
# pseudopotential that has been used for the QE calculations is stored.
parameters.targets.pseudopotential_path = "PATH_TO_YOUR_PSEUDOPOTENTIAL_FOLDER"

# These must be the same as in the data generation phase.
# For beryllium: ldos_gridoffset_ev=-5
# For aluminium: ldos_gridoffset_ev=-10
# The rest is the same.
parameters.targets.target_type = "LDOS"
parameters.targets.ldos_gridsize = 250
parameters.targets.ldos_gridspacing_ev = 0.1
parameters.targets.ldos_gridoffset_ev = -5
parameters.descriptors.descriptor_type = "SNAP"
parameters.descriptors.twojmax = 10
parameters.descriptors.rcutfac = 4.67637

# Compatbility with older SNAP data.
parameters.descriptors.snap_switchflag = 0


# The grid size can be obtained by, starting from the DFT grids, just scaling
# the grids up according to the number of atoms.
# This e.g. is the grid for 131072 beryllium atoms.
parameters.running.inference_data_grid = [1152, 576, 900]

# Build a data handler, but don't fill it with data (inference case).
inference_data_handler = mala.DataHandler(parameters,
                                          input_data_scaler=iscaler,
                                          output_data_scaler=oscaler)
inference_data_handler.clear_data()

# Read atomic positions for inference.
atoms = read("SOME_ATOMS")

predictor = mala.Predictor(parameters, network, inference_data_handler)
ldos_calculator: mala.LDOS = inference_data_handler.target_calculator

# Get critical values from a reference file (cutoff, temperature, etc.),
# ideally the DFT outputs of some training snapshot.
ldos_calculator. \
    read_additional_calculation_data("qe.out",
                                     "SOME_TRAINING_SNAPSHOT_OUTPUT.")

# Get the LDOS from the NN.
ldos = predictor.predict_for_atoms(atoms)

# Set the LDOS and calculate quantities as needed.
ldos_calculator.read_from_array(ldos, units="1/(Ry*Bohr^3)")

# Print the predicted total energy.
mala.printout(ldos_calculator.total_energy)

# Get the density, and save it to a file.
# Be careful, this can be RAM intensive,
# as the density is gathered to a single rank.
density = ldos_calculator.get_density(ldos, gather_density=True)

if get_rank() == 0:
    np.save("density.npy", density)
