"""
This script shows how MALA can be used to train surrogate models.

Can be used for both aluminium and beryllium data.
"""
import mala

# Set the parameters according to the Phys. Rev. B publication on MALA.
parameters = mala.Parameters()
parameters.data.input_rescaling_type = "feature-wise-standard"
parameters.data.output_rescaling_type = "normal"

# Set according to available hardware.
# For the models trained here, a single GPU should be enough.
parameters.use_gpu = True
parameters.data.use_lazy_loading = False

# LDOS parameters.
# For beryllium: ldos_gridoffset_ev=-5
# For aluminium: ldos_gridoffset_ev=-10
# The rest is the same.
parameters.targets.ldos_gridsize = 250
parameters.targets.ldos_gridspacing_ev = 0.1
parameters.targets.ldos_gridoffset_ev = -5

# Remaining training parameters.
parameters.running.early_stopping_epochs = 8
parameters.running.early_stopping_threshold = 0
parameters.running.learning_rate = 0.00001
parameters.running.trainingtype = "Adam"
parameters.running.learning_rate_scheduler = "ReduceLROnPlateau"
parameters.running.learning_rate_decay = 0.1
parameters.running.learning_rate_patience = 4
parameters.running.verbosity = True
parameters.running.max_number_epochs = 10000
parameters.running.mini_batch_size = 1000
parameters.network.layer_activations = ["LeakyReLU"]
parameters.running.checkpoints_each_epoch = 5

# Add data to the run.
# Filenames, paths, etc. of course depend on your machine.
# For beryllium, snapshot0 was used as training and snapshot2 as validation.
# For aluminium, it was snapshot0 and snapshot1.
base_path = "PATH_TO_YOUR_DATA"
snap_path = base_path+"snap/"
ldos_path = base_path+"ldos/"
out_path = base_path+"outputs/"
data_handler = mala.DataHandler(parameters)
data_handler.clear_data()
data_handler.add_snapshot("Be256_298K_snapshot0.in.npy", snap_path,
                          "Be256_298K_snapshot0.out.npy", ldos_path,
                          add_snapshot_as="tr")
data_handler.add_snapshot("Be256_298K_snapshot2.in.npy", snap_path,
                          "Be256_298K_snapshot2.out.npy", ldos_path,
                          calculation_output_file=out_path+"Be256_298K_snapshot2.out",
                          add_snapshot_as="va")
data_handler.prepare_data()

# Network layer sizes as given in the paper mentioned above.
parameters.network.layer_sizes = [data_handler.get_input_dimension(), 800,
                                 800, 800, 250,
                                 data_handler.get_output_dimension()]

network = mala.Network(parameters)
trainer = mala.Trainer(parameters, network, data_handler)
trainer.train_network()

# Save the run.
network.save_network("beryllium_model.network.pth")
parameters.save("beryllium_model.params.json")
data_handler.input_data_scaler.save("beryllium_model.iscaler.pkl")
data_handler.output_data_scaler.save("beryllium_model.oscaler.pkl")
